@@ -25,12 +25,18 @@ class SingleAxisTracking(object):
     rotation_limit: float = 45.0
     backtrack: bool = True
 
+
 @dataclass_json
 @dataclass
-class SystemDesign(object):
+class BaseSystemDesign(object):
     dc_capacity: float
     ac_capacity: float
     poi_limit: float
+
+
+@dataclass_json
+@dataclass
+class SystemDesign(BaseSystemDesign):
     gcr: float
     tracking: t.Union[FixedTilt, SingleAxisTracking]
     modules_per_string: t.Optional[int] = opt_field()
@@ -71,6 +77,7 @@ class ONDInverter(BaseInverter):
     dc_turn_on: float
     aux_loss: t.Optional[float] = opt_field()
     aux_loss_threshold: t.Optional[float] = opt_field()
+    includes_xfmr: t.Optional[bool] = True
     
     @classmethod
     def from_ond(cls, ond_file: str):
@@ -122,6 +129,7 @@ class Inverter(BaseInverter):
     c3: float
     vdcmax: float
     tdc: t.List[t.List[float]] = field(default_factory=lambda: [[1.0, 52.8, -0.021]])
+    includes_xfmr: t.Optional[bool] = True
 
 
 @dataclass_json
@@ -243,23 +251,44 @@ class PVModuleMermoudLejeune(object):
 
 @dataclass_json
 @dataclass
-class Losses(object):
-    ac_wiring: t.Optional[float] = opt_field()
+class Transformer(object):
+    rating: float
+    load_loss: float
+    no_load_loss: float
+
+
+@dataclass_json
+@dataclass
+class DCLosses(object):
     dc_optimizer: t.Optional[float] = opt_field()
     enable_snow_model: t.Optional[bool] = opt_field()
     dc_wiring: t.Optional[float] = opt_field()
+    soiling: t.Optional[t.List[float]] = opt_field()
     diodes_connections: t.Optional[float] = opt_field()
     mismatch: t.Optional[float] = opt_field()
     nameplate: t.Optional[float] = opt_field()
     rear_irradiance: t.Optional[float] = opt_field()
-    soiling: t.Optional[t.List[float]] = opt_field()
+    lid: t.Optional[float] = opt_field()
+    dc_array_adjustment: t.Optional[float] = opt_field()
+
+
+@dataclass_json
+@dataclass
+class ACLosses(object):
+    ac_wiring: t.Optional[float] = opt_field()
     tracking: t.Optional[float] = opt_field()
     transformer_load: t.Optional[float] = opt_field()
     transformer_no_load: t.Optional[float] = opt_field()
     transmission: t.Optional[float] = opt_field()
-    lid: t.Optional[float] = opt_field()
     poi_adjustment: t.Optional[float] = opt_field()
-    dc_array_adjustment: t.Optional[float] = opt_field()
+    mv_transformer: t.Optional[Transformer] = opt_field()
+
+
+@dataclass_json
+@dataclass
+class Losses(DCLosses, ACLosses):
+    pass
+
 
 @dataclass_json
 @dataclass
@@ -351,7 +380,56 @@ class PVModel(object):
     dc_degradation: t.Optional[float] = None
 
 
-DetailedPVModel = PVModel
+PVGenerationModel = DetailedPVModel = PVModel
+
+
+@dataclass_json
+@dataclass
+class DCProductionProfile(object):
+    power: t.List[float]
+    voltage: t.List[float]
+    ambient_temp: t.Optional[t.List[float]]
+
+
+@dataclass_json
+@dataclass
+class ACProductionProfile(object):
+    power: t.List[float]
+    ambient_temp: t.Optional[t.List[float]]
+
+
+@dataclass_json
+@dataclass
+class ACExternalGenerationModel(object):
+    production_override: ACProductionProfile
+    system_design: BaseSystemDesign
+    losses: ACLosses = opt_field()
+    project_term: t.Optional[int] = None
+
+
+@dataclass_json
+@dataclass
+class DCExternalGenerationModel(object):
+    production_override: DCProductionProfile
+    system_design: BaseSystemDesign
+    inverter: t.Union[Inverter, ONDInverter, str]
+    losses: DCLosses = opt_field()
+    project_term: t.Optional[int] = None
+
+
+GenerationModel = t.Union[PVGenerationModel, DCExternalGenerationModel, ACExternalGenerationModel]
+
+
+@dataclass_json
+@dataclass
+class BatteryHVAC(object):
+    container_temperature: float
+    cop: float
+    u_ambient: float
+    discharge_efficiency_container: float
+    charge_efficiency_container: float
+    aux_xfmr_efficiency: float
+
 
 @dataclass_json
 @dataclass
@@ -363,6 +441,8 @@ class Battery(object):
     degradation_rate: float
     term: int
     degradation_annual_cycles: t.Optional[float] = 365
+    hvac: t.Optional[BatteryHVAC] = opt_field()
+
 
 @dataclass_json
 @dataclass
@@ -374,6 +454,7 @@ class StorageInputs(object):
     step: t.Optional[int] = opt_field()
     flexible_solar: t.Optional[bool] = opt_field()
     dart: t.Optional[bool] = opt_field()
+    dam_annual_cycle_limit: t.Optional[float] = opt_field()
 
 
 @dataclass_json
@@ -426,19 +507,21 @@ class AncillaryMarkets(object):
 class StandaloneStorageModel(StorageModel):
     time_interval_mins: t.Optional[int] = opt_field()
     ancillary_markets: t.Optional[AncillaryMarkets] = opt_field()
+    ambient_temp: t.Optional[t.List[float]] = opt_field()
 
 
 @string_enum
 class StorageCoupling(Enum):
     ac = 'ac'
     dc = 'dc'
+    hv_ac = 'hv_ac'
 
 
 @dataclass_json
 @dataclass
 class PVStorageModel(object):
     storage_coupling: StorageCoupling = field(metadata=StorageCoupling.__metadata__)
-    pv_inputs: PVModel
+    pv_inputs: GenerationModel
     storage_inputs: StorageInputs
     energy_prices: t.Union[AncillaryEnergyPrices, t.List[float]]
     enable_grid_charge_year: t.Optional[int] = opt_field()
